/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * The SKU of the storage account.
 *
 */
class Sku {
  /**
   * Create a Sku.
   * @property {string} name Gets or sets the SKU name. Required for account
   * creation; optional for update. Note that in older versions, SKU name was
   * called accountType. Possible values include: 'Standard_LRS',
   * 'Standard_GRS', 'Standard_RAGRS', 'Standard_ZRS', 'Premium_LRS',
   * 'Premium_ZRS', 'Standard_GZRS', 'Standard_RAGZRS'
   * @property {string} [tier] Gets the SKU tier. This is based on the SKU
   * name. Possible values include: 'Standard', 'Premium'
   * @property {string} [resourceType] The type of the resource, usually it is
   * 'storageAccounts'.
   * @property {string} [kind] Indicates the type of storage account. Possible
   * values include: 'Storage', 'StorageV2', 'BlobStorage', 'FileStorage',
   * 'BlockBlobStorage'
   * @property {array} [locations] The set of locations that the SKU is
   * available. This will be supported and registered Azure Geo Regions (e.g.
   * West US, East US, Southeast Asia, etc.).
   * @property {array} [capabilities] The capability information in the
   * specified SKU, including file encryption, network ACLs, change
   * notification, etc.
   * @property {array} [restrictions] The restrictions because of which SKU
   * cannot be used. This is empty if there are no restrictions.
   */
  constructor() {
  }

  /**
   * Defines the metadata of Sku
   *
   * @returns {object} metadata of Sku
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Sku',
      type: {
        name: 'Composite',
        className: 'Sku',
        modelProperties: {
          name: {
            required: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          tier: {
            required: false,
            readOnly: true,
            serializedName: 'tier',
            type: {
              name: 'Enum',
              allowedValues: [ 'Standard', 'Premium' ]
            }
          },
          resourceType: {
            required: false,
            readOnly: true,
            serializedName: 'resourceType',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: false,
            readOnly: true,
            serializedName: 'kind',
            type: {
              name: 'String'
            }
          },
          locations: {
            required: false,
            readOnly: true,
            serializedName: 'locations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          capabilities: {
            required: false,
            readOnly: true,
            serializedName: 'capabilities',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'SKUCapabilityElementType',
                  type: {
                    name: 'Composite',
                    className: 'SKUCapability'
                  }
              }
            }
          },
          restrictions: {
            required: false,
            serializedName: 'restrictions',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'RestrictionElementType',
                  type: {
                    name: 'Composite',
                    className: 'Restriction'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = Sku;
